@@ -9,7 +9,9 @@ from .promise import Promise
 from .processor import Processor
 from .flag import Flag
 from .gate import Gate
-
+from .LogFile import LogFile, LogStream
+from .producer import Producer
+from .escrow import Escrow
 
 __version__ = Version
 version_info = tuple(Version.split("."))
@@ -30,5 +32,5 @@ __a_ll__ = [
     'Timeout',
     'Promise',
     'Scheduler',
-    'Gate'
+    'Gate', 'LogFile', 'LogStream'
 ]
